#!/usr/bin/env python3

import shapefile
import argparse
from os import path

parser = argparse.ArgumentParser()
parser.add_argument("input_dir", help = "containing the three input shapefiles")
parser.add_argument("output_dir",
                    help = "containing the three filtered shapefiles")
parser.add_argument("--renumber", help = "reset indices of eddies",
                    action = "store_true")
args = parser.parse_args()

with shapefile.Reader(path.join(args.input_dir, "extremum")) as extremum, \
     shapefile.Reader(path.join(args.input_dir, "outermost_contour")) \
     as outermost_cont, \
     shapefile.Reader(path.join(args.input_dir, "max_speed_contour")) \
     as max_speed_cont, \
     shapefile.Writer(path.join(args.output_dir, "extremum")) \
     as extremum_filt, \
     shapefile.Writer(path.join(args.output_dir, "outermost_contour")) \
     as outermost_cont_filt, \
     shapefile.Writer(path.join(args.output_dir, "max_speed_contour")) \
     as max_speed_cont_filt:
    extremum_filt.fields = extremum.fields[1:]
    outermost_cont_filt.fields = outermost_cont.fields[1:]
    max_speed_cont_filt.fields = max_speed_cont.fields[1:]
    if args.renumber: eddy_index = 0

    for shape_rec_extr, shape_rec_outer, shape_rec_max \
        in zip(extremum, outermost_cont, max_speed_cont):
        if 6 <= shape_rec_extr.shape.points[0][0] <= 12 \
           and - 33 <= shape_rec_extr.shape.points[0][1] <= - 27:
            if args.renumber: eddy_index += 1

            for sr, w in zip([shape_rec_extr, shape_rec_outer, shape_rec_max],
                             [extremum_filt, outermost_cont_filt,
                              max_speed_cont_filt]):
                r = sr.record
                if args.renumber: r["eddy_index"] = eddy_index
                w.record(*r)
                w.shape(sr.shape)
