program test_overlap

  use, intrinsic:: ISO_FORTRAN_ENV

  ! Libraries:
  use jumble, only: new_unit, get_command_arg_dyn
  use nr_util, only: deg_to_rad, assert
  use shapelib, only: shpfileobject, shpclose
  use shapelib_03, only: shp_open_03

  use derived_types, only: snapshot
  use init_shapefiles_m, only: init_shapefiles
  use read_field_indices_m, only: read_field_indices
  use read_snapshot_m, only: read_snapshot
  use overlap_m, only: overlap
  use write_overlap_m, only: init_interpolated_eddy

  implicit none

  character(len = :), allocatable:: snapshot_1, snapshot_2
  integer unit_edgelist, i, k, copy
  type(snapshot), allocatable:: flow(:) ! (max_delta + 1)
  TYPE(shpfileobject) hshp_extremum ! shapefile extremum
  TYPE(shpfileobject) hshp_outermost ! shapefile outermost_contour
  TYPE(shpfileobject) hshp_max_speed ! shapefile max_speed_contour

  real:: corner_deg(2) = [huge(0.), huge(0.)], corner(2)
  ! longitude and latitude of the corner of the whole grid, in degrees
  ! and in rad

  real:: step_deg(2) = [0.25, 0.25], step(2)
  ! longitude and latitude steps, in degrees and in rad

  logical periodic ! grid is periodic in longitude
  integer:: nlon = - 1, nlat = - 1, max_delta = 1

  integer:: dist_lim = 12
  ! We look for an overlapping eddy at dist_lim (in grid points) of
  ! the first extremum.

  namelist /main_nml/ corner_deg, step_deg, nlon, nlat, dist_lim, max_delta

  !-------------------------------------------------------------------------

  call assert(command_argument_count() == 2, &
       "Required arguments: snapshot-directory snapshot-directory")
  call get_command_arg_dyn(1, snapshot_1)
  call get_command_arg_dyn(2, snapshot_2)

  write(unit = error_unit, nml = main_nml)
  write(unit = error_unit, fmt = *) "Enter namelist main_nml."
  read(unit = *, nml = main_nml)
  write(unit = *, nml = main_nml)

  ! As we are requiring the grid spacing to be uniform, the value of
  ! "periodic" may be deduced from the values of step_deg(1) and nlon:
  periodic = nint(360. / step_deg(1)) == nlon
  print *, "periodic = ", periodic
  if (periodic) call assert(2 * dist_lim * step_deg(1) < 180., &
       "test_overlap dist_lim")
  copy = merge(dist_lim, 0, periodic)
  
  corner = corner_deg * deg_to_rad
  step = step_deg * deg_to_rad
  allocate(flow(max_delta + 1))

  call shp_open_03(hshp_extremum, trim(snapshot_1) // "/extremum", &
       pszaccess = "rb")
  call shp_open_03(hshp_outermost, trim(snapshot_1) // "/outermost_contour", &
       pszaccess = "rb")
  call shp_open_03(hshp_max_speed, trim(snapshot_1) // "/max_speed_contour", &
       pszaccess = "rb")
  call read_field_indices(hshp_extremum, hshp_outermost, hshp_max_speed)
  call read_snapshot(flow(1), k, hshp_extremum, hshp_outermost, &
       hshp_max_speed, corner, step, nlon, nlat, copy)
  CALL shpclose(hshp_extremum)
  CALL shpclose(hshp_outermost)
  CALL shpclose(hshp_max_speed)
  print *, snapshot_1, ", k (not used) = ", k
  print *, "Enter flow(1)%list_vis%delta_out:"
  read *, flow(1)%list_vis%delta_out
  print *, flow(1)%list_vis%delta_out

  call shp_open_03(hshp_extremum, trim(snapshot_2) // "/extremum", &
       pszaccess = "rb")
  call shp_open_03(hshp_outermost, trim(snapshot_2) // "/outermost_contour", &
       pszaccess = "rb")
  call shp_open_03(hshp_max_speed, trim(snapshot_2) // "/max_speed_contour", &
       pszaccess = "rb")
  ! We assume that the field indices are the same than in snapshot_1.
  call read_snapshot(flow(max_delta + 1), k, hshp_extremum, hshp_outermost, &
       hshp_max_speed, corner, step, nlon, nlat, copy)
  CALL shpclose(hshp_extremum)
  CALL shpclose(hshp_outermost)
  CALL shpclose(hshp_max_speed)
  print *, snapshot_2, ", k = ", k
  print *, "Enter flow(max_delta + 1)%list_vis%delta_in:"
  read *, flow(max_delta + 1)%list_vis%delta_in
  print *, flow(max_delta + 1)%list_vis%delta_in

  flow(2:max_delta)%number_eddies = 20000

  call new_unit(unit_edgelist)
  open(unit_edgelist, file = "edgelist.csv", status = "replace", &
       action  = "write")

  ! Title lines:
  write(unit_edgelist, fmt = "(1x, a)") '"predecessor date subscript" ' &
       // '"predecessor eddy subscript" "successor date subscript" ' &
       // '"successor eddy subscript"'
  write(unit_edgelist, fmt = *) "k1 i1 k2 i2 weight"

  call init_shapefiles(hshp_extremum, hshp_outermost, hshp_max_speed)
  call init_interpolated_eddy
  call overlap(flow, unit_edgelist, nlon, nlat, periodic, dist_lim, &
       hshp_extremum, hshp_outermost, hshp_max_speed, j = max_delta + 1, &
       k = k, delta = max_delta)
  close(unit_edgelist)
  print *, 'Created file "edgelist.csv".'
  CALL shpclose(hshp_extremum)
  print *, 'Created shapefile "SHP_triplet/extremum".'
  CALL shpclose(hshp_outermost)
  print *, 'Created shapefile "SHP_triplet/outermost_contour".'
  CALL shpclose(hshp_max_speed)
  print *, 'Created shapefile "SHP_triplet/max_speed_contour".'

  print *, snapshot_1, ":"
  print *, "Valid isolated eddies:"
  do i = 1, flow(1)%number_vis_extr
     if (flow(1)%list_vis(i)%valid .and. flow(1)%list_vis(i)%delta_out &
          == huge(0)) write(unit = *, fmt = "(i0, 1x)", advance = "no") i
  end do
  print *

  print *
  print *, Snapshot_2, ":"
  print *, "Valid isolated eddies:"

  do i = 1, flow(max_delta + 1)%number_vis_extr
     if (flow(max_delta + 1)%list_vis(i)%valid &
          .and. flow(max_delta + 1)%list_vis(i)%delta_in == huge(0)) &
          write(unit = *, fmt = "(i0, 1x)", advance = "no") i
  end do
  print *

end program test_overlap
