program test_send_recv

  use, intrinsic:: ISO_FORTRAN_ENV

  ! Libraries:
  use ezmpi, only: ezmpi_bcast
  use mpi_f08, only: mpi_init, mpi_finalize, MPI_Comm_rank, MPI_Comm_world, &
       MPI_Comm_size, mpi_abort, MPI_TAG_UB, MPI_Comm_get_attr, &
       MPI_ADDRESS_KIND, MPI_io, MPI_ANY_SOURCE
  use nr_util, only: deg_to_rad, assert
  use shapelib, only: shpfileobject, shpclose
  use shapelib_03, only: shp_open_03

  use derived_types, only: snapshot
  use init_shapefiles_m, only: init_shapefiles
  use read_field_indices_m, only: read_field_indices
  use read_snapshot_m, only: read_snapshot
  use recv_snapshot_m, only: recv_snapshot
  use send_snapshot_m, only: send_snapshot
  use write_eddy_m, only: write_eddy
  use write_snapshot_m, only: write_snapshot

  implicit none

  type(snapshot) s
  integer rank, n_proc, k, copy
  logical flag
  INTEGER(KIND=MPI_ADDRESS_KIND) attribute_val

  real:: corner(2) = [0.125, - 59.875]
  ! longitude and latitude of the corner of the whole grid, in degrees

  real:: step(2) = [0.25, 0.25] ! longitude and latitude steps, in degrees
  integer:: nlon = 120, nlat = 120

  integer:: dist_lim = 12
  ! We look for an overlapping eddy at dist_lim (in grid points) of
  ! the first extremum.

  namelist /main_nml/ corner, step, nlon, nlat, dist_lim
  logical periodic ! grid is periodic in longitude
  TYPE(shpfileobject) hshp_extremum ! shapefile extremum
  TYPE(shpfileobject) hshp_outermost ! shapefile outermost_contour
  TYPE(shpfileobject) hshp_max_speed ! shapefile max_speed_contour

  !---------------------------------------------------------------------

  call mpi_init
  call MPI_Comm_rank(MPI_Comm_world, rank)
  call MPI_Comm_size(MPI_Comm_world, n_proc)

  if (n_proc /= 2) then
     if (rank == 0) print *, "test_send_recv: exactly 2 processes required"
     call mpi_abort(MPI_Comm_world, errorcode = 1)
  end if

  if (rank == 1) then
     call MPI_Comm_get_attr(MPI_Comm_world, MPI_io, attribute_val, flag)
     call assert(flag, "test_send_recv MPI_Comm_get_attr MPI_io")
     call assert(attribute_val == MPI_ANY_SOURCE, &
          "test_send_recv MPI_io MPI_ANY_SOURCE")
     
     write(unit = error_unit, nml = main_nml)
     write(unit = error_unit, fmt = *) "Enter namelist main_nml."
     read(unit = *, nml = main_nml)
     write(unit = *, nml = main_nml)

     ! As we are requiring the grid spacing to be uniform, the value of
     ! "periodic" may be deduced from the values of step(1) and nlon:
     periodic = nint(360. / step(1)) == nlon
     print *, "periodic = ", periodic
     if (periodic) call assert(2 * dist_lim * step(1) < 180., &
          "test_send_recv dist_lim")
     copy = merge(dist_lim, 0, periodic)
  end if

  call ezmpi_bcast(corner, root = 1)
  call ezmpi_bcast(step, root = 1)
  call ezmpi_bcast(nlon, root = 1)
  call ezmpi_bcast(nlat, root = 1)
  call ezmpi_bcast(copy, root = 1)

  if (rank == 1) then
     call shp_open_03(hshp_extremum, "SHP_triplet_old/extremum", &
          pszaccess = "rb")
     call shp_open_03(hshp_outermost, "SHP_triplet_old/outermost_contour", &
          pszaccess = "rb")
     call shp_open_03(hshp_max_speed, "SHP_triplet_old/max_speed_contour", &
          pszaccess = "rb")
     call read_field_indices(hshp_extremum, hshp_outermost, hshp_max_speed)
     call read_snapshot(s, k, hshp_extremum, hshp_outermost, hshp_max_speed, &
          corner * deg_to_rad, step * deg_to_rad, nlon, nlat, copy)
     CALL shpclose(hshp_extremum)
     CALL shpclose(hshp_outermost)
     CALL shpclose(hshp_max_speed)
     call send_snapshot(s, dest = 0, tag = k)
  else
     ! rank == 0
     call MPI_Comm_get_attr(MPI_Comm_world, MPI_TAG_UB, attribute_val, flag)
     call assert(flag, "test_send_recv MPI_Comm_get_attr MPI_TAG_UB")
     print *, "MPI_TAG_UB = ", attribute_val
     call recv_snapshot(s, nlon, nlat, copy, source = 1, tag = 20454)
     call write_snapshot(s, corner, step, nlon, nlat, copy, k = 20454)
  end if

  call mpi_finalize

end program test_send_recv
