module write_snapshot_m

  implicit none

contains

  subroutine write_snapshot(s, corner, step, nlon, nlat, copy, k)

    ! Libraries:
    use nr_util, only: arth
    use shapelib, only: shpfileobject, shpclose

    use derived_types, only: snapshot
    use init_shapefiles_m, only: init_shapefiles
    use write_eddy_m, only: write_eddy
    use write_extr_map_m, only: write_extr_map

    type(snapshot), intent(in):: s
    
    real, intent(in):: corner(:) ! (2)
    ! longitude and latitude of the corner of the whole grid in input
    ! NetCDF, in degrees

    real, intent(in):: step(:) ! (2) ! longitude and latitude steps, in degrees

    integer, intent(in):: nlon, nlat
    ! size of ssh array in input NetCDF, assuming no repeated point if
    ! the grid is global

    integer, intent(in):: copy, k

    ! Local:
    TYPE(shpfileobject) hshp_extremum ! shapefile extremum
    TYPE(shpfileobject) hshp_outermost ! shapefile outermost_contour
    TYPE(shpfileobject) hshp_max_speed ! shapefile max_speed_contour
    integer i

    !-----------------------------------------------------------------

    call init_shapefiles(hshp_extremum, hshp_outermost, hshp_max_speed)

    ! Write snapshot:
    do i = 1, s%number_vis_extr
       call write_eddy(s%list_vis(i), hshp_extremum, hshp_outermost, &
            hshp_max_speed, k, i)
    end do

    CALL shpclose(hshp_extremum)
    CALL shpclose(hshp_outermost)
    CALL shpclose(hshp_max_speed)
    print *, 'Created shapefiles in SHP_triplet.'

    print *, "Number of extrema:", s%number_vis_extr
    print *, "Number of eddies:", s%number_eddies
    
    print *, "s%ind_extr:"

    do i = 1, s%number_vis_extr
       print *, s%ind_extr(:, i)
    end do

    call write_extr_map(s%extr_map, longitude = arth(corner(1) - copy &
         * step(1), step(1), nlon + 2 * copy), latitude = arth(corner(2), &
         step(2), nlat))

  end subroutine write_snapshot

end module write_snapshot_m
