module get_snapshot_m

  implicit none

contains

  subroutine get_snapshot(s, m, n_proc, k_end, max_delta, nlon, nlat, k, &
       corner, step, periodic, dist_lim)

    ! Libraries:
    use shapelib, only: shpfileobject, shpclose
    use shapelib_03, only: shp_open_03

    use derived_types, only: snapshot
    use read_snapshot_m, only: read_snapshot
    use receive_snapshot_m, only: receive_snapshot

    type(snapshot), intent(out):: s

    integer, intent(in):: m ! rank of MPI process
    integer, intent(in):: n_proc ! number of MPI processes
    integer, intent(in):: k_end ! last date index analyzed by this MPI process

    integer, intent(in):: max_delta
    ! maximum interval of date indices between which we look for
    ! overlapping of eddies

    integer, intent(in):: nlon, nlat
    integer, intent(in):: k ! date index

    real, intent(in):: corner(:) ! (2) longitude and latitude of the
    ! corner of the whole grid, in rad

    real, intent(in):: step(:) ! (2) longitude and latitude steps, in rad
    logical, intent(in):: periodic ! grid is periodic in longitude

    integer, intent(in):: dist_lim
    ! We look for an overlapping eddy at dist_lim (in grid points) of
    ! the first extremum.

    ! Local:
    TYPE(shpfileobject) hshp_extremum ! shapefile extremum
    TYPE(shpfileobject) hshp_outermost ! shapefile outermost_contour
    TYPE(shpfileobject) hshp_max_speed ! shapefile max_speed_contour
    integer k2
    character(len = 5) directory

    !--------------------------------------------------------------

    if (m == n_proc .or. k <= k_end - max_delta) then
       ! Read at date k:
       write(unit = directory, fmt = *) k
       call shp_open_03(trim(directory) // "extremum", pszaccess = "rb", &
            hshp = hshp_extremum)
       call shp_open_03(trim(directory) // "outermost_contour", &
            pszaccess = "rb", hshp = hshp_outermost)
       call shp_open_03(trim(directory) // "max_speed_contour", &
            pszaccess = "rb", hshp = hshp_max_speed)
       call read_snapshot(s, k2, hshp_extremum, hshp_outermost, &
            hshp_max_speed, corner, step, nlon, nlat, periodic, dist_lim)
       call assert(k2 == k, "get_snapshot: k2 == k")
       CALL shpclose(hshp_extremum)
       CALL shpclose(hshp_outermost)
       CALL shpclose(hshp_max_speed)
    else
       call receive_snapshot(s, source = m + 1, tag = k)
    end if

  end subroutine get_snapshot

end module get_snapshot_m
