module init_shapefiles_m

  implicit none

  integer, protected:: ifield_extr_ssh, ifield_extr_date, &
       ifield_extr_eddy_index, ifield_extr_interp, ifield_extr_cycl, &
       ifield_extr_valid, ifield_extr_speed
  integer, protected:: ifield_out_r_eq_area, ifield_out_ssh, &
       ifield_out_date, ifield_out_eddy_index, ifield_out_radius4
  integer, protected:: ifield_max_speed_r_eq_area, ifield_max_speed_ssh, &
       ifield_max_speed_date, ifield_max_speed_eddy_index
  
contains

  subroutine init_shapefiles(hshp_extremum, hshp_outermost, hshp_max_speed)

    use shapelib, only: shpt_point, shpt_polygon, shpfileobject, ftdouble, &
         ftinteger
    use shapelib_03, only: shp_create_03, dbf_add_field_03

    TYPE(shpfileobject), intent(out):: hshp_extremum ! shapefile extremum

    TYPE(shpfileobject), intent(out):: hshp_outermost
    ! shapefile outermost_contour

    TYPE(shpfileobject), intent(out):: hshp_max_speed
    ! shapefile max_speed_contour

    !---------------------------------------------------------------------

    call shp_create_03("SHP_triplet/extremum", shpt_point, hshp_extremum)
    call dbf_add_field_03(ifield_extr_ssh, hshp_extremum, 'ssh', ftdouble, &
         nwidth = 13, ndecimals = 6)
    call dbf_add_field_03(ifield_extr_date, hshp_extremum, 'date_index', &
         ftinteger, nwidth = 5, ndecimals = 0)
    call dbf_add_field_03(ifield_extr_eddy_index, hshp_extremum, 'eddy_index', &
         ftinteger, nwidth = 5, ndecimals = 0)
    call dbf_add_field_03(ifield_extr_interp, hshp_extremum, 'interpolat', &
         ftinteger, nwidth = 1, ndecimals = 0)
    call dbf_add_field_03(ifield_extr_cycl, hshp_extremum, 'cyclone', &
         ftinteger, nwidth = 1, ndecimals = 0)
    call dbf_add_field_03(ifield_extr_valid, hshp_extremum, 'valid', &
         ftinteger, nwidth = 1, ndecimals = 0)
    call dbf_add_field_03(ifield_extr_speed, hshp_extremum, 'speed', ftdouble, &
         nwidth = 13, ndecimals = 6)

    call shp_create_03("SHP_triplet/outermost_contour", shpt_polygon, &
         hshp_outermost)
    call dbf_add_field_03(ifield_out_r_eq_area, hshp_outermost, &
         'r_eq_area', ftdouble, nwidth = 10, ndecimals = 4)
    call dbf_add_field_03(ifield_out_ssh, hshp_outermost, 'ssh', ftdouble, &
         nwidth = 13, ndecimals = 6)
    call dbf_add_field_03(ifield_out_date, hshp_outermost, 'date_index', &
         ftinteger, nwidth = 5, ndecimals = 0)
    call dbf_add_field_03(ifield_out_eddy_index, hshp_outermost, 'eddy_index', &
         ftinteger, nwidth = 5, ndecimals = 0)
    call dbf_add_field_03(ifield_out_radius4, hshp_outermost, 'radius4', &
         ftinteger, nwidth = 2, ndecimals = 0)

    call shp_create_03("SHP_triplet/max_speed_contour", shpt_polygon, &
         hshp_max_speed)
    call dbf_add_field_03(ifield_max_speed_r_eq_area, hshp_max_speed, &
         'r_eq_area', ftdouble, nwidth = 10, ndecimals = 4)
    call dbf_add_field_03(ifield_max_speed_ssh, hshp_max_speed, 'ssh', &
         ftdouble, nwidth = 13, ndecimals = 6)
    call dbf_add_field_03(ifield_max_speed_date, hshp_max_speed, 'date_index', &
         ftinteger, nwidth = 5, ndecimals = 0)
    call dbf_add_field_03(ifield_max_speed_eddy_index, hshp_max_speed, &
         'eddy_index', ftinteger, nwidth = 5, ndecimals = 0)

  end subroutine init_shapefiles

end module init_shapefiles_m
